; mozctlx.def : Declares the module parameters.   
                                                   
LIBRARY      "mozctlx.dll"                        
EXPORTS                                            
    ; ActiveX exports                              
    DllCanUnloadNow        @100 PRIVATE            
    DllGetClassObject      @101 PRIVATE            
    DllRegisterServer      @102 PRIVATE            
    DllUnregisterServer    @103 PRIVATE            
                                                   
