// EditToolBar.cpp : implementation file
//

#include "stdafx.h"
#include "cbrowse.h"
#include "EditToolBar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEditToolBar

CEditToolBar::CEditToolBar()
{
}

CEditToolBar::~CEditToolBar()
{
}


BEGIN_MESSAGE_MAP(CEditToolBar, CToolBarCtrl)
	//{{AFX_MSG_MAP(CEditToolBar)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEditToolBar message handlers
