/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XForms support.
 *
 * The Initial Developer of the Original Code is
 * Novell, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Allan Beaufour <abeaufour@novell.com>
 *  David Landwehr <dlandwehr@novell.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsXFormsNodeState.h"

void
nsXFormsNodeState::Set(PRUint16 aFlags, PRBool aVal)
{
  if (aVal) {
    mState |= aFlags;
  } else {
    mState &= ~aFlags;
  }
}

nsXFormsNodeState&
nsXFormsNodeState::operator&=(PRUint16 aMask)
{
  mState &= aMask;
  return *this;
}

PRBool
nsXFormsNodeState::operator==(nsXFormsNodeState& aCmp) const
{
  return (mState == aCmp.mState) ? PR_TRUE : PR_FALSE;
}

PRBool
nsXFormsNodeState::TestAndClear(eFlag_t aFlag)
{
  PRUint16 test = mState & aFlag;
  mState &= ~aFlag;
  return test ? PR_TRUE : PR_FALSE;
}

PRBool
nsXFormsNodeState::TestAndSet(eFlag_t aFlag)
{
  PRUint16 test = mState & aFlag;
  mState |= aFlag;
  return test ? PR_TRUE : PR_FALSE;
}

PRBool
nsXFormsNodeState::Test(eFlag_t aFlag) const
{
  return (mState & aFlag) ? PR_TRUE : PR_FALSE;
}

#ifdef DEBUG_XF_NODESTATE
const char* flagnames[] = {0,
                           "eFlag_READONLY",
                           "eFlag_CONSTRAINT",
                           "eFlag_RELEVANT",
                           "eFlag_REQUIRED",
                           "eFlag_SCHEMA_VALID",
                           "eFlag_VALID",
                           "eFlag_INHERITED_RELEVANT",
                           "eFlag_INHERITED_READONLY",
                           "eFlag_DISPATCH_VALUE_CHANGED",
                           "eFlag_DISPATCH_READONLY_CHANGED",
                           "eFlag_DISPATCH_VALID_CHANGED",
                           "eFlag_DISPATCH_RELEVANT_CHANGED",
                           "eFlag_DISPATCH_REQUIRED_CHANGED",
                           "eFlag_DISPATCH_CONSTRAINT_CHANGED",
                           0};



#include <stdio.h>

void
nsXFormsNodeState::PrintFlags() const
{
  int pos = 1;
  while (flagnames[pos]) {
    if (mState & (1 << pos)) {
      printf("\t%s\n", flagnames[pos]);
    }
    ++pos;
  }
}
#endif
