LIBRARY         NSLDAPPR32V50
DESCRIPTION     'LDAP to NSPR glue library for 32-bit Windows'
VERSION		5.0
HEAPSIZE        4096

EXPORTS
; we need to manually assign ordinal numbers so we can add new routines
; and not disturb the ordinals and thus not require callers to relink.
; 
; exports list (generated by genexports.pl)
; 
	prldap_init		@2010
	prldap_install_routines		@2011
	prldap_set_session_info		@2012
	prldap_get_session_info		@2013
	prldap_set_socket_info		@2014
	prldap_get_socket_info		@2015
	prldap_set_session_option		@2016
	prldap_get_session_option		@2017
; 
; end of generated exports list.
