@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: find toolbar ::::: */

#find-closebutton {
  padding-right: 4px;
  list-style-image: url("chrome://global/skin/icons/closetab.png") !important;
  list-style-image: none;
  border: none;
}

#find-closebutton {
  background-image: none !important;
}

#find-closebutton:hover:active {
	list-style-image: url("chrome://global/skin/icons/closetab-active.png") !important;
}

#FindToolbar {
	background-image: url("chrome://global/skin/icons/white-gray-gradient.gif");
	background-repeat: repeat-x;
	background-position: bottom right;
	background-color: rgb(246, 246, 246);
        border-top: 1px solid #b3b3b3;
	border-bottom: none;
	list-style-image: url("chrome://global/skin/icons/find.png");
}

#FindToolbar > toolbarbutton {
  padding-left: 7px;
}

#FindToolbar > label {
  font-weight: bold;
  color: #565656;
}

#FindToolbar > toolbarbutton > .toolbarbutton-icon 
{
  height: 16px !important;
  padding: 0;
}

#FindToolbar > toolbarbutton > .toolbarbutton-text, #FindToolbar > checkbox > .checkbox-label-box {
  height: 16px !important;
  padding: 1px 7px 1px 2px;
  font-weight: bold;
  color: #565656;
}

#FindToolbar .checkbox-icon, #find-closebutton > .toolbarbutton-text  {
  display: none;  
}

#FindToolbar .checkbox-check {
  -moz-appearance: checkbox-small;
  margin-left: 7px;
}

#FindToolbar .checkbox-label {
  padding-bottom: 0;
}

#find-field {
  background: url("chrome://global/skin/icons/find-bar-background.png") no-repeat;
  -moz-appearance: none;
  height: 22px !important;
  width: 182px !important;
  border: none !important;
  margin: 2px 4px;
  padding: 3px 14px 3px 24px;
}

#FindToolbar[flash="true"] > #find-field-container > #find-field {
  background: url("chrome://global/skin/icons/find-bar-flash.png") no-repeat;
}

#find-field[status="notfound"] {
  background: url("chrome://global/skin/icons/find-bar-notfound.png") no-repeat;
  color: #FFF;
}

/* ::::: find ::::: */

#find-previous 
{
  list-style-image: url("chrome://global/skin/icons/find.png");
  -moz-image-region: rect(0px 16px 16px 0px);
}

#find-previous[disabled="true"]
{
  opacity: 0.5;
}
	
#find-previous:hover
{
  -moz-image-region: rect(0px 32px 16px 16px);
}

#find-next 
{
  list-style-image: url("chrome://global/skin/icons/find.png");
  -moz-image-region: rect(16px 16px 32px 0px);
}

#find-next[disabled="true"]
{
  opacity: 0.5;
}

#find-next:hover
{
  -moz-image-region: rect(16px 32px 32px 16px);
}

#highlight {
  -moz-image-region: rect(0px 48px 16px 32px);
}

#highlight:hover {
  -moz-image-region: rect(16px 48px 32px 32px);
}

#highlight[disabled="true"] {
  opacity: 0.5;
}

#highlight:active, #highlight[checked="true"] {
  -moz-image-region: rect(48px 48px 64px 32px);
}

#highlight[checked="true"]:hover {
  -moz-image-region: rect(64px 48px 80px 32px);
}

#find-status-icon
{
  margin-left: 12px;
  width: 16px;
  height: 16px;
}
/* find-next button */

#find-next {
  -moz-image-region: rect(0px 16px 16px 0px);
}

#find-next:hover {
  -moz-image-region: rect(16px 16px 32px 0px);
}

#find-next[disabled="true"] {
  -moz-image-region: rect(32px 16px 48px 0px) !important;
}

/* find-previous button */

#find-previous {
  -moz-image-region: rect(0px 32px 16px 16px);
}

#find-previous:hover {
  -moz-image-region: rect(16px 32px 32px 16px);
}

#find-previous[disabled="true"] {
  -moz-image-region: rect(32px 32px 48px 16px) !important;
}

/* highlight button */

#highlight {
  -moz-image-region: rect(0px 48px 16px 32px);
}

#highlight:hover {
  -moz-image-region: rect(16px 48px 32px 32px);
}

#highlight[disabled="true"] {
  -moz-image-region: rect(32px 48px 48px 32px) !important;
}

#highlight:active, #highlight[checked="true"] {
  -moz-image-region: rect(48px 48px 64px 32px);
}

#highlight[checked="true"]:hover {
  -moz-image-region: rect(64px 48px 80px 32px);
}

#find-status-icon {
  display: none;
}

#find-status {
	color: #436599 !important;
}

